/*
 * Copyright (c) 2014 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.framework.util.ui.widgets.traceview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.EventListener;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.swing.BoxLayout;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionListener;

import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.traceview.FilteredListModel.ListModelFilter;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.DefaultEvent;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.DefaultTrace;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Event;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Trace;

/**
 * A {@link JPanel} that displays a list of arbitrary objects that are
 * visualized as "wedges" using {@link ProMTraceView}. It uses a {@link JList}
 * under the hood, which can be accessed directly using {@link #getList()}.
 * <p>
 * To use this component you need to create a {@link TraceBuilder} that takes
 * any of your objects and returns a {@link Trace} of {@link Event}s. This
 * conversion happens on-the-fly and only for the objects that are currently
 * shown to the user. Therefore, the performance overhead should be small.
 * <p>
 * There are default implementations {@link DefaultTrace} and
 * {@link DefaultEvent} available for convenience. If your class already
 * implements {@link Trace}, then use the provided {@link NoOpTraceBuilder}.
 * <p>
 * It is possible to sort {@link #sort(Comparator)} and filter
 * {@link #filter(ListModelFilter)} the list.
 * <p>
 * It is also possible to listen for click, selection and mouse movement events
 * on an element of the list by using
 * {@link #addTraceClickListener(ClickListener)}.
 * {@link #addTraceSelectionListener(ListSelectionListener)}, as well as
 * {@link #addTraceMoveListener(MoveListener)}.
 * 
 * @author F. Mannhardt
 * 
 * @param <T>
 *            the class of objects that are to be visualized
 */
public class ProMTraceList<T> extends JPanel {

	public interface ClickListener<T> extends EventListener {

		/**
		 * Event that the user has double clicked on a trace.
		 * 
		 * @param trace
		 *            the object that has been clicked on
		 * @param traceIndex
		 *            the index of the trace in the list
		 * @param eventIndex
		 *            the index of the event that has been clicked on or -1 in
		 *            case not event was below the mouse pointer
		 * @param e
		 *            the original {@link MouseEvent}
		 */
		public void traceMouseDoubleClicked(T trace, int traceIndex, int eventIndex, MouseEvent e);

		/**
		 * Event that the user has clicked on a trace.
		 * 
		 * @param trace
		 *            the object that has been clicked on
		 * @param traceIndex
		 *            the index of the trace in the list
		 * @param eventIndex
		 *            the index of the event that has been clicked on or -1 in
		 *            case not event was below the mouse pointer
		 * @param e
		 *            the original {@link MouseEvent}
		 */
		public void traceMouseClicked(T trace, int traceIndex, int eventIndex, MouseEvent e);

	}

	public interface MoveListener<T> extends EventListener {

		/**
		 * Event that the user has moved the mouse over a trace.
		 * 
		 * @param trace
		 *            the object that has been moved over
		 * @param traceIndex
		 *            the index of the trace in the list
		 * @param eventIndex
		 *            the index of the event or -1 in case no event was below
		 *            the mouse pointer
		 * @param e
		 *            the original {@link MouseEvent}
		 */
		public void traceMouseMoved(T trace, int traceIndex, int eventIndex, MouseEvent e);

	}

	/**
	 * Builds {@link Trace} objects for object of class T.
	 * 
	 * @author F. Mannhardt
	 * 
	 * @param <T>
	 *            class of object that is converted to a {@link Trace}
	 */
	public interface TraceBuilder<T> {

		/**
		 * @param element
		 * @return a {@link Trace}
		 */
		Trace<? extends Event> build(T element);

	}

	/**
	 * Can be used to override the default coloring of an {@link Event} based on
	 * some other information. If only one of the colors needs to be overridden,
	 * please use {@link DefaultWedgeBuilder}.
	 * 
	 * @author F. Mannhardt
	 * 
	 */
	public interface WedgeBuilder {

		Color buildNameColor(Trace<? extends Event> trace);

		Color buildInfoColor(Trace<? extends Event> trace);

		Color buildWedgeColor(Trace<? extends Event> trace, Event event);

		Integer assignWedgeGap(Trace<? extends Event> trace, Event event);

		Stroke buildBorderStroke(Trace<? extends Event> trace, Event event);

		Color buildBorderColor(Trace<? extends Event> trace, Event event);

		Color buildLabelColor(Trace<? extends Event> trace, Event event);

		Color buildTopLabelColor(Trace<? extends Event> trace, Event event);

		Color buildBottomLabelColor(Trace<? extends Event> trace, Event event);

		Color buildBottom2LabelColor(Trace<? extends Event> trace, Event event);

	}

	/**
	 * {@link WedgeBuilder} that simply returns the original color of an
	 * {@link Event}. Override single methods to change the color.
	 * 
	 * @author F. Mannhardt
	 * 
	 */
	public static class DefaultWedgeBuilder implements WedgeBuilder {

		public Color buildWedgeColor(Trace<? extends Event> trace, Event event) {
			return event.getWedgeColor();
		}

		public Color buildLabelColor(Trace<? extends Event> trace, Event event) {
			return event.getLabelColor();
		}

		public Color buildTopLabelColor(Trace<? extends Event> trace, Event event) {
			return event.getTopLabelColor();
		}

		public Color buildBottomLabelColor(Trace<? extends Event> trace, Event event) {
			return event.getBottomLabelColor();
		}

		public Color buildBottom2LabelColor(Trace<? extends Event> trace, Event event) {
			return event.getBottomLabel2Color();
		}

		public Color buildBorderColor(Trace<? extends Event> trace, Event event) {
			return event.getBorderColor();
		}

		public Integer assignWedgeGap(Trace<? extends Event> trace, Event event) {
			return null;
		}

		public Stroke buildBorderStroke(Trace<? extends Event> trace, Event e) {
			return null;
		}

		public Color buildNameColor(Trace<? extends Event> trace) {
			return trace.getNameColor();
		}

		public Color buildInfoColor(Trace<? extends Event> trace) {
			return trace.getInfoColor();
		}

	}

	/**
	 * {@link WedgeBuilder} that delegates all methods to another
	 * {@link WedgeBuilder}.
	 * 
	 * @author F. Mannhardt
	 * 
	 */
	public static class DelegateWedgeBuilder extends DefaultWedgeBuilder {

		private final WedgeBuilder wedgeBuilder;

		public DelegateWedgeBuilder(WedgeBuilder wedgeBuilder) {
			this.wedgeBuilder = wedgeBuilder;
		}

		public Color buildWedgeColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildWedgeColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildWedgeColor(trace, event);
		}

		public Color buildLabelColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildLabelColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildLabelColor(trace, event);
		}

		public Color buildTopLabelColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildTopLabelColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildTopLabelColor(trace, event);
		}

		public Color buildBottomLabelColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildBottomLabelColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildBottomLabelColor(trace, event);
		}

		public Color buildBottom2LabelColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildBottom2LabelColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildBottom2LabelColor(trace, event);
		}

		public Color buildBorderColor(Trace<? extends Event> trace, Event event) {
			Color color = wedgeBuilder.buildBorderColor(trace, event);
			if (color != null) {
				return color;
			}
			return super.buildBorderColor(trace, event);
		}

		public Integer assignWedgeGap(Trace<? extends Event> trace, Event event) {
			Integer wedgeGap = wedgeBuilder.assignWedgeGap(trace, event);
			if (wedgeGap != null) {
				return wedgeGap;
			}
			return super.assignWedgeGap(trace, event);
		}

		public Stroke buildBorderStroke(Trace<? extends Event> trace, Event e) {
			Stroke wedgeStroke = wedgeBuilder.buildBorderStroke(trace, e);
			if (wedgeStroke != null) {
				return wedgeStroke;
			}
			return super.buildBorderStroke(trace, e);
		}

		public Color buildNameColor(Trace<? extends Event> trace) {
			Color color = wedgeBuilder.buildNameColor(trace);
			if (color != null) {
				return color;
			}
			return super.buildNameColor(trace);
		}

		public Color buildInfoColor(Trace<? extends Event> trace) {
			Color color = wedgeBuilder.buildInfoColor(trace);
			if (color != null) {
				return color;
			}
			return super.buildInfoColor(trace);
		}

	}

	/**
	 * TraceBuilder that just returns a class, which already implements
	 * {@link Trace}.
	 * 
	 * @author F. Mannhardt
	 * 
	 */
	public static class NoOpTraceBuilder<T extends Event> implements TraceBuilder<Trace<T>> {

		public Trace<? extends Event> build(Trace<T> e) {
			return e;
		}

	}

	private static final Font DEFAULT_FONT = new Font(null, Font.PLAIN, 10);

	private final static class ProMTraceListMouseAdapter<T> extends MouseAdapter {

		private final Set<ClickListener<T>> clickListener;
		private final Set<MoveListener<T>> moveListener;
		private final JList<T> list;
		private ProMTraceViewCellRenderer<T> cellRenderer;

		private ProMTraceListMouseAdapter(JList<T> list, ProMTraceViewCellRenderer<T> cellRenderer,
				Set<ClickListener<T>> clickListener, Set<MoveListener<T>> moveListener) {
			super();
			this.list = list;
			this.cellRenderer = cellRenderer;
			this.clickListener = clickListener;
			this.moveListener = moveListener;
		}

		public void mouseClicked(MouseEvent e) {

			if (!clickListener.isEmpty()) {
				final int traceIndex = getTraceIndex(e);
				if (traceIndex != -1) {
					final T trace = list.getModel().getElementAt(traceIndex);
					final int eventIndex = getEventIndex(traceIndex, trace, e);

					for (ClickListener<T> listener : clickListener) {
						if (e.getClickCount() == 2) {
							listener.traceMouseDoubleClicked(trace, traceIndex, eventIndex, e);
						} else if (e.getClickCount() == 1) {
							listener.traceMouseClicked(trace, traceIndex, eventIndex, e);
						}
					}
				}
			}
		}

		public void mouseMoved(MouseEvent e) {

			if (!moveListener.isEmpty()) {
				final int traceIndex = getTraceIndex(e);
				if (traceIndex != -1) {
					final T trace = list.getModel().getElementAt(traceIndex);
					final int eventIndex = getEventIndex(traceIndex, trace, e);

					for (MoveListener<T> listener : moveListener) {
						listener.traceMouseMoved(trace, traceIndex, eventIndex, e);
					}
				}
			}
		}

		private int getEventIndex(int traceIndex, T trace, MouseEvent e) {
			return cellRenderer.translateToEventIndex(e.getPoint(), trace, list.isSelectedIndex(traceIndex));
		}

		private int getTraceIndex(MouseEvent e) {
			Rectangle r = list.getCellBounds(0, list.getLastVisibleIndex());
			if (r != null && r.contains(e.getPoint())) {
				return list.locationToIndex(e.getPoint());
			}
			return -1;
		}

	}

	/**
	 * The {@link ListCellRenderer} used to paint the objects.
	 * 
	 * @author F. Mannhardt
	 * 
	 * @param <T>
	 */
	@SuppressWarnings("rawtypes")
	private static class ProMTraceViewCellRenderer<E> extends ProMTraceView implements ListCellRenderer {

		private static final long serialVersionUID = -2495069999724478333L;

		private TraceBuilder<E> traceBuilder;
		private Object currentValue;
		private int fixedWidthLimit = 500;

		public ProMTraceViewCellRenderer(ProMTraceList<E> traceList, TraceBuilder<E> traceBuilder, Font defaultFont) {
			super(defaultFont);
			setTraceBuilder(traceBuilder);
			setOpaque(false);
		}

		@SuppressWarnings("unchecked")
		// Hudson Java 1.6 workaround
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
				boolean cellHasFocus) {
			if (list.getModel().getSize() > getFixedWidthLimit()) {
				setFixedWedgeWidth(getMaxWedgeWidth());
			}
			if (value != currentValue) {
				setTrace(getTraceBuilder().build((E) value));
				updatePreferredSize();
			}
			currentValue = value;
			setSelected(isSelected);
			setForeground(list.getForeground());
			return this;
		}

		public int translateToEventIndex(Point point, E value, boolean isSelected) {
			if (value != currentValue) {
				setTrace(getTraceBuilder().build(value));
				updatePreferredSize();
			}
			currentValue = value;
			setSelected(isSelected);
			return translateToEventIndex(point);
		}

		public TraceBuilder<E> getTraceBuilder() {
			return traceBuilder;
		}

		public void setTraceBuilder(TraceBuilder<E> traceBuilder) {
			this.currentValue = null;
			this.traceBuilder = traceBuilder;
		}

		public int getFixedWidthLimit() {
			return fixedWidthLimit;
		}

		public void setFixedWidthLimit(int fixedWidthLimit) {
			this.currentValue = null;
			this.fixedWidthLimit = fixedWidthLimit;
		}

	}

	private static final long serialVersionUID = -8729322696443726936L;

	private final FilteredListModel<SortableListModel<T>, T> listModel;
	@SuppressWarnings("rawtypes")
	// Because of Java 1.6
	private final JList jList;
	private final ProMTraceViewCellRenderer<T> cellRenderer;
	private final JScrollPane scrollPane;

	private transient CopyOnWriteArraySet<ClickListener<T>> clickListener = new CopyOnWriteArraySet<ClickListener<T>>();
	private transient CopyOnWriteArraySet<MoveListener<T>> moveListener = new CopyOnWriteArraySet<MoveListener<T>>();

	private JProgressBar progressBar;

	public ProMTraceList(TraceBuilder<T> traceBuilder) {
		this(Collections.<T>emptyList(), traceBuilder);
	}

	public ProMTraceList(Collection<T> traces, TraceBuilder<T> traceBuilder) {
		this(traces, traceBuilder, DEFAULT_FONT);
	}

	public ProMTraceList(Collection<T> traces, TraceBuilder<T> traceBuilder, Font labelFont) {
		this(traces, traceBuilder, null, labelFont);
	}

	public ProMTraceList(Collection<T> traces, TraceBuilder<T> traceBuilder, Comparator<T> order) {
		this(traces, traceBuilder, order, DEFAULT_FONT);
	}

	public ProMTraceList(Collection<T> traces, TraceBuilder<T> traceBuilder, Comparator<T> order, Font labelFont) {
		this(new SortableListModelImpl<>(traces), traceBuilder, order, labelFont);
	}

	public ProMTraceList(SortableListModel<T> traceModel, TraceBuilder<T> traceBuilder) {
		this(traceModel, traceBuilder, null, DEFAULT_FONT);
	}

	public ProMTraceList(SortableListModel<T> traceModel, TraceBuilder<T> traceBuilder, Comparator<T> order) {
		this(traceModel, traceBuilder, order, DEFAULT_FONT);
	}

	/**
	 * @param traces
	 * @param traceBuilder
	 * @param order
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	// Because of Java 1.6
	public ProMTraceList(SortableListModel<T> traceModel, TraceBuilder<T> traceBuilder, Comparator<T> order,
			Font labelFont) {
		listModel = new FilteredListModel<>(traceModel);
		if (order != null) {
			listModel.getOriginalListModel().sort(order);
		}
		cellRenderer = new ProMTraceViewCellRenderer<>(this, traceBuilder, labelFont);

		jList = new JList(getListModel());
		jList.setFixedCellHeight((int) cellRenderer.getPreferredSize().getHeight());
		jList.setOpaque(false);
		jList.setForeground(null);
		jList.setBackground(null);
		jList.setCellRenderer(cellRenderer);

		ProMTraceListMouseAdapter mouseAdapter = new ProMTraceListMouseAdapter(jList, cellRenderer, clickListener,
				moveListener);
		jList.addMouseListener(mouseAdapter);
		jList.addMouseMotionListener(mouseAdapter);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setOpaque(false);
		setForeground(null);
		setBackground(null);

		scrollPane = new ProMScrollPane(jList);
		scrollPane.setOpaque(false);
		scrollPane.setForeground(null);
		scrollPane.setBackground(null);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setForeground(null);
		scrollPane.getViewport().setBackground(null);
		
		progressBar = new JProgressBar();
		progressBar.setIndeterminate(true);
		progressBar.setString("Loading ...");
		progressBar.setStringPainted(true);
		
		add(scrollPane, BorderLayout.CENTER);
	}

	public void addTraceSelectionListener(ListSelectionListener l) {
		jList.addListSelectionListener(l);
	}

	public void removeTraceSelectionListener(ListSelectionListener l) {
		jList.removeListSelectionListener(l);
	}

	public void addTraceClickListener(ClickListener<T> l) {
		clickListener.add(l);
	}

	public void removeTraceClickListener(ClickListener<T> l) {
		clickListener.remove(l);
	}

	public void addTraceMoveListener(MoveListener<T> l) {
		moveListener.add(l);
	}

	public void removeTraceMoveListener(MoveListener<T> l) {
		moveListener.remove(l);
	}

	public void sort(Comparator<T> ordering) {
		getListModel().getOriginalListModel().sort(ordering);
	}

	public void filter(ListModelFilter<T> filter) {
		getListModel().setFilter(filter);
		getListModel().doFilter();
	}

	@SuppressWarnings("rawtypes")
	// Because of Java 1.6
	public JList getList() {
		return jList;
	}

	/**
	 * @return
	 */
	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	/**
	 * @return a {@link FilteredListModel} of the used {@link SortableListModel}
	 */
	public FilteredListModel<SortableListModel<T>, T> getListModel() {
		return listModel;
	}

	public void add(T element) {
		listModel.getOriginalListModel().add(element);
	}

	public void addAll(Iterable<T> elements) {
		listModel.getOriginalListModel().addAll(elements);
	}

	/**
	 * Clears the underlying data structure {@link #getListModel()}.
	 */
	public void clear() {
		listModel.getOriginalListModel().clear();
	}

	/**
	 * Set the factor by which the color of the unselected traces is modified.
	 * For example, 1.0 will result in no change and 0.5 will darken the color.
	 * 
	 * @param attenuationFactor
	 *            non-negative factor
	 */
	public void setAttenuationFactor(float attenuationFactor) {
		cellRenderer.setAttenuationFactor(attenuationFactor);
	}

	public float getAttenuationFactor() {
		return cellRenderer.getAttenuationFactor();
	}

	public WedgeBuilder getWedgeBuilder() {
		return cellRenderer.getWedgeBuilder();
	}

	/**
	 * Use a {@link WedgeBuilder} to dynamically override the color of an
	 * {@link Event}. Use {@link DefaultWedgeBuilder}, if you only want to
	 * override a certain color.
	 * 
	 * @param wedgeBuilder
	 *            that overrides the default color of an event
	 */
	public void setWedgeBuilder(WedgeBuilder wedgeBuilder) {
		if (wedgeBuilder == null) {
			cellRenderer.setWedgeBuilder(new DefaultWedgeBuilder());
		} else {
			cellRenderer.setWedgeBuilder(new DelegateWedgeBuilder(wedgeBuilder));
		}
	}

	public int getMaxWedgeWidth() {
		return cellRenderer.getMaxWedgeWidth();
	}

	/**
	 * Set the maximum width (in pixel) of a single "event wedge".
	 * 
	 * @param maxWidth
	 */
	public void setMaxWedgeWidth(int maxWidth) {
		cellRenderer.setMaxWedgeWidth(maxWidth);
	}
	
	/**
	 * Set the fixed width (in pixel) of a single "event wedge". Set to -1 to
	 * enable dynamic sizing (slower) according to the length of labels.
	 * 
	 * @param fixedWidth
	 */
	public void setFixedWedgeWidth(int fixedWidth) {
		cellRenderer.setFixedWedgeWidth(fixedWidth);
	}
	
	public int getFixedWedgeLimit() {
		return cellRenderer.getFixedWidthLimit();
	}

	/**
	 * Changes the built-in limit of events per trace that is used to determine
	 * when to switch to fixed size wedge width for performance reasons.
	 * 
	 * @param fixedWedgeLimit
	 */
	public void setFixedWedgeLimit(int fixedWedgeLimit) {
		cellRenderer.setFixedWidthLimit(fixedWedgeLimit);
	}

	public int getCollapsedLabelLength() {
		return cellRenderer.getCollapsedLabelLength();
	}

	/**
	 * Set the length of the label shown in a collapsed event, in an unselected
	 * trace.
	 * 
	 * @param collapsedLabelLength
	 */
	public void setCollapsedLabelLength(int collapsedLabelLength) {
		cellRenderer.setCollapsedLabelLength(collapsedLabelLength);
	}

	public Stroke getWedgeStroke() {
		return cellRenderer.getWedgeStroke();
	}

	/**
	 * @param wedgeStroke
	 *            the default {@link Stroke} that the wedges border is painted
	 *            with
	 */
	public void setWedgeStroke(Stroke wedgeStroke) {
		cellRenderer.setWedgeStroke(wedgeStroke);
	}

	public int getWedgeGap() {
		return cellRenderer.getWedgeGap();
	}

	/**
	 * @param wedgeGap
	 *            sets the default gap between two event wedges
	 */
	public void setWedgeGap(int wedgeGap) {
		cellRenderer.setWedgeGap(wedgeGap);
	}

	public TraceBuilder<T> getTraceBuilder() {
		return cellRenderer.getTraceBuilder();
	}

	/**
	 * @param traceBuilder
	 *            that is used to create traces out of the input objects
	 */
	public void setTraceBuilder(TraceBuilder<T> traceBuilder) {
		cellRenderer.setTraceBuilder(traceBuilder);
	}

	public void beforeUpdate() {
		setEnabled(false);
		remove(scrollPane);
		add(progressBar, BorderLayout.CENTER);
		validate();
	}

	public void afterUpdate() {
		remove(progressBar);
		add(scrollPane, BorderLayout.CENTER);
		validate();
		setEnabled(true);
	}

}
